﻿
Public Class Form1

    ' ============================================================================================
    '  Constants and Variables 
    ' ============================================================================================
    Private Strike As Class_StrikeDetector
    Private Scope As Class_DataScope
    Private Slots As ThereminoSlots = New ThereminoSlots()

    ' ============================================================================================
    '   User interface events 
    ' ============================================================================================
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' --------------------------------------------
        If OperatingSystemIsWindows Then
            Me.Visible = False
            Me.Opacity = 0
            Form2.Opacity = 0
        End If
        ' --------------------------------------------
        Load_INI()
        Load_Config()
        StartThereminoHAL()
        SetLocales()
        UpdateUserInterface()
        Text = AppTitleAndVersion("Theremino Meteo")
        'MeterBar1 = New MeterBar(PictureBox1, 4)
        'MeterBar2 = New MeterBar(PictureBox2, 5)
        Strike = New Class_StrikeDetector
        Scope = New Class_DataScope(pbox_ScrollingScope, pbox_Scale)
        ' ------------------------------------------- FadeIn
        Form2.Show(Me)
        Form2.Visible = True
        Me.Visible = True
        If OperatingSystemIsWindows Then
            Forms_FadeTo(1, 400)
        End If
        ' ------------------------------------------- Timers
        ShedulerMaxPrecision()
        UiTimerFast.Interval = 1 ' 1 means 60 Hz in sync with scheduler
        UiTimerFast.Start()
        UiTimerSlow.Interval = 30
        UiTimerSlow.Start()
        WorkerThread_Start()
        ' -------------------------------------------
        EventsAreEnabled = True
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        UiTimerSlow.Stop()
        UiTimerFast.Stop()
        WorkerThread.Abort()
        ShedulerDefaultPrecision()
        Forms_FadeTo(0, 500)
        Me.Refresh()
        Save_INI()
        Save_Config()
        StopThereminoHAL()
    End Sub

    Private Sub Form1_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Return
        LimitFormPosition(Me)
    End Sub

    Private Sub UpdateUserInterface()
        InitControls()
        Refresh()
    End Sub

    Private Sub btn_EditChannels_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_EditChannels.ClickButtonArea
        Form2.Visible = True
    End Sub

    ' =========================================================================
    '  Worker Thread
    ' ========================================================================= 
    Dim WorkerThread As Threading.Thread
    Sub WorkerThread_Start()
        ' --------------------------------------------------------- asynchronously start the Thread 
        WorkerThread = New Threading.Thread(AddressOf WorkerThread_Execute)
        ' --------------------------------------------------------- make the thread As background thread.
        WorkerThread.IsBackground = True
        ' --------------------------------------------------------- Set the Priority of the thread.
        WorkerThread.Priority = Threading.ThreadPriority.Highest
        ' --------------------------------------------------------- start the thread.
        WorkerThread.Start()
    End Sub
    Sub WorkerThread_Execute()
        Do
            WorkerThread_Tick()
            Threading.Thread.Sleep(1)
        Loop Until False
    End Sub

    ' =========================================================================
    '  WINDOWS SCHEDULER PRECISION
    ' ========================================================================= 
    Private Declare Function timeBeginPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
    Private Declare Function timeEndPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
    Private Sub ShedulerMaxPrecision()
        If OperatingSystemIsWindows Then
            timeBeginPeriod(1)
        End If
    End Sub
    Private Sub ShedulerDefaultPrecision()
        If OperatingSystemIsWindows Then
            timeEndPeriod(1)
        End If
    End Sub


    ' =========================================================================
    '   THEREMINO HAL - START STOP -
    ' =========================================================================
    Private HalProcess As Process = Nothing
    Private Sub StartThereminoHAL()
        Dim path As String = PlatformAdjustedFileName(Application.StartupPath & "\Theremino_HAL.exe")
        If Not My.Computer.FileSystem.FileExists(path) Then
            path = PlatformAdjustedFileName(Application.StartupPath & "\Theremino_HAL\Theremino_HAL.exe")
        End If
        If Not My.Computer.FileSystem.FileExists(path) Then Return
        Dim psi As New ProcessStartInfo
        psi.WorkingDirectory = IO.Path.GetDirectoryName(path)
        psi.FileName = path
        HalProcess = Process.Start(psi)
    End Sub
    Private Sub StopThereminoHAL()
        If HalProcess IsNot Nothing AndAlso Not HalProcess.HasExited Then
            Try
                HalProcess.CloseMainWindow()
            Finally
            End Try
        End If
    End Sub


    ' ===================================================================================
    '   MenuStrip and ToolStrip Gradients
    ' ===================================================================================
    Private Sub MenuStrip1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MenuStrip1.Paint
        Dim bounds As New Rectangle(0, 0, _
                                    MenuStrip1.Width, MenuStrip1.Height)
        'Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
        '                                               Color.White, _
        '                                               Color.FromArgb(200, 200, 200), _
        '                                               Drawing2D.LinearGradientMode.Horizontal)

        Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
                                                       Color.White, _
                                                           Color.FromArgb(200, 200, 200), _
                                                           Drawing2D.LinearGradientMode.Vertical)
        e.Graphics.FillRectangle(brush, bounds)
    End Sub
    'Private Sub ToolStrip1_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles ToolStrip1.Paint
    '    Dim bounds As New Rectangle(0, 0, _
    '                                ToolStrip1.Width, ToolStrip1.Height)
    '    Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
    '                                                   Color.White, _
    '                                                   Color.FromArgb(200, 200, 200), _
    '                                                   Drawing2D.LinearGradientMode.Vertical)
    '    e.Graphics.FillRectangle(brush, bounds)
    'End Sub

    ' ===================================================================================
    '  MenuStrip and ToolStrip accepting the first click
    '  If the form receives a WM_PARENTNOTIFY (528) message and is not focused 
    '  then the form is activated before to exec the message
    ' ===================================================================================
    Protected Overrides Sub WndProc(ByRef m As Message)
        If m.Msg = 528 AndAlso Not Me.Focused Then
            Me.Activate()
        End If
        MyBase.WndProc(m)
    End Sub



    ' =======================================================================================
    '   MENU FILE
    ' =======================================================================================
    Private Sub Menu_File_Exit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_File_Exit.Click
        Me.Close()
    End Sub

    ' =======================================================================================
    '   MENU LANGUAGE
    ' =======================================================================================
    Private Sub Menu_Language_DropDownOpening(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language.DropDownOpening
        For Each item As ToolStripMenuItem In Menu_Language.DropDownItems
            If item.Name.EndsWith(Language, StringComparison.InvariantCultureIgnoreCase) Then
                item.Select()
            End If
        Next
    End Sub
    Private Sub Menu_Language_DEU_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_DEU.Click
        Language = "DEU"
        SetLocales()
        Save_INI()
        UpdateUserInterface()
    End Sub
    Private Sub Menu_Language_ENG_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_ENG.Click
        Language = "ENG"
        SetLocales()
        Save_INI()
        UpdateUserInterface()
    End Sub
    Private Sub Menu_Language_ESP_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_ESP.Click
        Language = "ESP"
        SetLocales()
        Save_INI()
        UpdateUserInterface()
    End Sub
    Private Sub Menu_Language_FRA_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_FRA.Click
        Language = "FRA"
        SetLocales()
        Save_INI()
        UpdateUserInterface()
    End Sub
    Private Sub Menu_Language_ITA_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Menu_Language_ITA.Click
        Language = "ITA"
        SetLocales()
        Save_INI()
        UpdateUserInterface()
    End Sub
    Private Sub Menu_Language_JPN_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_Language_JPN.Click
        Language = "JPN"
        SetLocales()
        Save_INI()
        UpdateUserInterface()
    End Sub

    ' =======================================================================================
    '   MENU HELP
    ' =======================================================================================
    Private Sub Menu_Help_ProgramHelp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_Help_ProgramHelp.Click
        OpenLocalizedHelp("Theremino_Meteo_Help", ".pdf")
    End Sub
    Private Sub Menu_Help_Hardware_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_Help_HardwareInfo.Click
        OpenLocalizedHelp("Theremino_Meteo_HardwareInfo", ".pdf")
    End Sub
    Private Sub Menu_Help_OpenProgramFolder_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_Help_OpenProgramFolder.Click
        Process.Start(Application.StartupPath)
    End Sub
    Private Sub OpenLocalizedHelp(ByVal name As String, Optional ByVal ext As String = ".rtf")
        Dim fname As String = PlatformAdjustedFileName(Application.StartupPath & "\Docs\" & name & "_" & Language & ext)
        If FileExists(fname) Then
            Process.Start(fname)
        Else
            fname = PlatformAdjustedFileName(Application.StartupPath & "\Docs\" & name & "_ENG" & ext)
            If FileExists(fname) Then
                Process.Start(fname)
            Else
                fname = PlatformAdjustedFileName(Application.StartupPath & "\Docs\" & name & "_ITA" & ext)
                If FileExists(fname) Then
                    Process.Start(fname)
                Else
                    fname = PlatformAdjustedFileName(Application.StartupPath & "\Docs\" & name & ext)
                    If FileExists(fname) Then
                        Process.Start(fname)
                    End If
                End If
            End If
        End If
    End Sub

    ' =======================================================================================
    '   MENU ABOUT
    ' =======================================================================================
    Private Sub Menu_About_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Menu_About.Click
        Form_About.ShowDialog(Me)
    End Sub


    ' ============================================================================================
    '  CONTROLS
    ' ============================================================================================
    Private ColorBack As Color = Color.Cornsilk
    Private ColorBackChecked As Color = Color.Yellow
    Private ColorText As Color = Color.FromArgb(40, 60, 20)
    Private OldNumChannels As Int32 = 0

    Friend Sub InitControls()
        If OperatingSystemIsWindows Then
            SuspendRedraw(Me)
        End If
        Dim scrollpos As Point = Panel1.AutoScrollPosition
        scrollpos.Y = Math.Abs(scrollpos.Y)
        Panel1.AutoScroll = False
        If NumChannels >= OldNumChannels Then
            For i As Int32 = OldNumChannels + 1 To NumChannels
                AddChannelControls(i)
            Next
        Else
            For i As Int32 = OldNumChannels To NumChannels + 1 Step -1
                RemoveChannelControls(i)
            Next
        End If
        OldNumChannels = NumChannels
        For i As Int32 = 1 To NumChannels
            SetChannelControlProps(i)
        Next
        Panel1.AutoScroll = True
        Panel1.AutoScrollPosition = scrollpos
        If OperatingSystemIsWindows Then
            ResumeRedraw(Me)
        End If
    End Sub

    Private Sub SetChannelControlProps(ByVal i As Int32)
        Dim PosY As Int32 = 4 + (i - 1) * 54
        ' ------------------------------------------------- BtnName
        With ChannelArray(i).btn_Name
            'Dim c() As Color = {Color.AliceBlue, Color.AntiqueWhite}
            'Dim p() As Single = {0, 1}
            '.ColorFillBlend = New cBlendItems(c, p)
            .Tag = i
            .CheckButton = True
            .Corners = New CornersProperty(0, 0, 0, 0)
            .Font = New Font("Tahoma", 14, FontStyle.Regular)
            .Text = GetChannelNameLocalized(i)
            .TextAlign = ContentAlignment.MiddleCenter
            .TextMargin = New Padding(0)
            .TextShadow = Color.Transparent
            .TextShadowChecked = Color.Transparent
            .BorderShow = True
            .BorderColor = Color.Gray
            .ForeColor = ColorText
            .ForeColorChecked = ColorText
            .BackColor = Color.Transparent
            .ColorFillSolid = ColorBack
            .ColorFillSolidChecked = ColorBackChecked
            .FillType = MyButton.eFillType.Solid
            .FillTypeChecked = MyButton.eFillType.Solid
            .Location = New Point(5, PosY)
            .Size = New Size(136, 52)
        End With
        ' ------------------------------------------------- TxtValue
        With ChannelArray(i).txt_Value
            .Tag = i
            .AutoSize = False
            .ReadOnly = True
            .Multiline = False
            .Font = New Font("Tahoma", 22, FontStyle.Regular)
            .Text = "- - -"
            .TextAlign = HorizontalAlignment.Center
            .BorderStyle = BorderStyle.FixedSingle
            .ForeColor = ColorText
            .BackColor = ColorBack
            .Location = New Point(145, PosY)
            .Size = New Size(200, 52)
        End With
    End Sub

    Private Sub AddChannelControls(ByVal i As Int32)
        Dim PosY As Int32 = 4 + i * 48
        ' ------------------------------------------------- BtnName
        ChannelArray(i).btn_Name = New MyButton
        AddHandler ChannelArray(i).btn_Name.CheckedChanged, AddressOf BtnNameCheckedChanged_Handler
        Me.Panel1.Controls.Add(ChannelArray(i).btn_Name)
        ' ------------------------------------------------- CmbType
        ChannelArray(i).txt_Value = New TextBox
        Me.Panel1.Controls.Add(ChannelArray(i).txt_Value)
    End Sub

    Private Sub RemoveChannelControls(ByVal i As Int32)
        ' ------------------------------------------------- BtnName
        RemoveHandler ChannelArray(i).btn_Name.CheckedChanged, AddressOf BtnNameCheckedChanged_Handler
        Me.Panel1.Controls.Remove(ChannelArray(i).btn_Name)
        ChannelArray(i).btn_Name = Nothing
        ' ------------------------------------------------- TxtValue
        Me.Panel1.Controls.Remove(ChannelArray(i).txt_Value)
        ChannelArray(i).txt_Value = Nothing
    End Sub

    Private Sub BtnNameCheckedChanged_Handler(ByVal sender As Object, ByVal e As EventArgs)
        Dim c As MyButton = CType(sender, MyButton)
        Dim n As Int32 = CInt(Val(c.Tag))
    End Sub


    ' =======================================================================================
    '   PARAMS
    ' =======================================================================================

    ' SENSOR	 mV/Gauss        mV/Gauss          Max range
    '           (5Volt supply)  (3.3Volt supply)   (Gauss)
    '--------------------------------------------------------
    ' A1321	    5.0 mV/G        3.30 mV/G          +/- 500
    ' A1322	    3.1 mV/G        2.05 mV/G          +/- 800
    ' A1323	    2.5 mV/G        1.65 mV/G          +/- 1000
    ' A1301		2.5 mV/G        1.650 mV/G         +/- 1000
    ' A1302	    1.3 mV/G        0.858 mV/G         +/- 1900

    ' Earth magnetic field
    ' -------------------------------------------------------
    ' World 0.23 to 0.66 Gauss
    ' Italy 0.45 to 0.48 Gauss


    Private Function DistanceToString(ByVal DistanceKm As Single) As String
        If DistanceKm > 300 Then
            Return "- - -"
        ElseIf DistanceKm >= 10 Then
            Return DistanceKm.ToString("0", Globalization.CultureInfo.InvariantCulture) & " Km"
        ElseIf DistanceKm >= 1 Then
            Return DistanceKm.ToString("0.0", Globalization.CultureInfo.InvariantCulture) & " Km"
        Else
            Return (DistanceKm * 1000).ToString("0", Globalization.CultureInfo.InvariantCulture) & " m"
        End If
    End Function

    ' TODO ---
    Private Sub UiTimerSlow_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UiTimerSlow.Tick
        If Not EventsAreEnabled Then Return
        Static LockObject As Object = New Object
        SyncLock LockObject
            Dim CI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture
            Dim v As Single
            For i As Int32 = 1 To NumChannels
                With ChannelArray(i)
                    If .txt_Value.Focused AndAlso .txt_Value.SelectionLength > 0 Then Continue For
                    v = Slots.ReadSlot_NoNan(.Slot)
                    Select Case .ChannelType
                        Case ChannelTypes.Strikes
                            Strike.StrikeFifo.CalcLastStrikes(60)
                            .txt_Value.Text = DistanceToString(Strike.StrikeFifo.MeanDistanceKm)
                            If .btn_Name.Checked Then
                                Label_Details_1b.Text = Strike.GetPeakValue.ToString("0")
                                Label_Details_2b.Text = Strike.StrikeFifo.StrikesPerMinute.ToString("0")
                                Label_Details_3b.Text = DistanceToString(Strike.StrikeFifo.MinDistanceKm)
                                Label_Details_4b.Text = DistanceToString(Strike.StrikeFifo.MeanDistanceKm)
                            End If
                            Continue For
                        Case Else
                            v *= .TrimSlope
                            v += .TrimZero
                            .txt_Value.Text = v.ToString(.Format, CI) & " " & .Unit
                    End Select
                End With
            Next
        End SyncLock
    End Sub

    Private Sub UiTimerFast_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UiTimerFast.Tick
        If Not EventsAreEnabled Then Return
        Dim v As Single = Strike.GetPeakValue
        If v > 0 Then Scope.SetV2(v)

        Scope.SetV1(Strike.MinValue)
        Scope.Display(50)
    End Sub

    Private Sub WorkerThread_Tick()
        For i As Int32 = 1 To NumChannels
            With ChannelArray(i)
                Select Case .ChannelType
                    Case ChannelTypes.Strikes
                        Strike.Update(Slots.ReadSlot_NoNan(.Slot))
                        'if .btn_Name.Checked
                End Select
            End With
        Next
    End Sub


End Class