﻿
Public Class Form2

    ' ===================================================================================
    '  MenuStrip, ToolStrip and RichTextBox accepting the first click
    '  If the form receives a WM_PARENTNOTIFY (528) message and is not focused 
    '  then the form is activated before to exec the message
    ' ===================================================================================
    Protected Overrides Sub WndProc(ByRef m As Message)
        If m.Msg = 528 AndAlso Not Me.Focused Then
            Me.Activate()
        End If
        MyBase.WndProc(m)
    End Sub

    Private Color_NormalText As Color = Color.Black
    Private Color_UnusedText As Color = Color.FromArgb(60, 60, 80)
    Private Color_ErrorBack As Color = Color.FromArgb(255, 200, 180)
    Private Color_TextBack As Color = Color.FromArgb(255, 240, 190)
    Private Color_Back As Color = Color.FromArgb(250, 240, 240)

    Private Sub Form2_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Text = AppTitleAndVersion("Theremino Meteo") & " - Channels"
        ' --------------------------------------------------------------------
        With RTB
            '.AcceptsTab = True
            .DetectUrls = False
            .WordWrap = False
            .HideSelection = False
            .Font = New Font("Courier New", 10)
            .ZoomFactor = 1
        End With
        TextToChannelArray()
    End Sub

    Private Sub Form2_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Not e.CloseReason = CloseReason.FormOwnerClosing Then
            Me.Visible = False
            e.Cancel = True
        End If
    End Sub

    Private Sub Form2_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Return
        LimitFormPosition(Me)
    End Sub

    Private Sub RTB1_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles RTB.TextChanged
        If Not EventsAreEnabled Then Exit Sub
        TextToChannelArray()
    End Sub

    Friend Sub TextToChannelArray()
        If OperatingSystemIsWindows Then
            RTB.BackColor = Color_Back
            SuspendRedraw(RTB)
            RTB_SaveSelectionAndScroll()
            ' ----------------------------------------------------- unused color for all
            RTB.SelectAll()
            RTB.SelectionBackColor = RTB.BackColor
            RTB.SelectionColor = Color_UnusedText
        End If
        ' -----------------------------------------------------
        NumChannels = 0
        For i As Int32 = 0 To RTB.Lines.Length - 1
            Dim l As String = RTB.Lines(i)
            If OperatingSystemIsWindows Then
                RTB_SelectLine(i)
                RTB.SelectionBackColor = Color_TextBack
                RTB.SelectionColor = Color_NormalText
            End If
            With ChannelArray(NumChannels)
                Select Case ExtractParamName(l).ToLowerInvariant
                    Case "channeltype"
                        NumChannels += 1
                        With ChannelArray(NumChannels)
                            .ChannelType = TypeFromString(l)
                            ' ---------------------------------------------------------- ChannelType defaults 
                            Select Case .ChannelType
                                Case ChannelTypes.AmbientalNoise : .Format = "0 dB"
                                Case ChannelTypes.Dust : .Format = "0.00 g/m3"
                                Case ChannelTypes.Humidity : .Format = "0.0 %"
                                Case ChannelTypes.Light : .Format = "0.0 W/m2"
                                Case ChannelTypes.MagneticField : .Format = "0.00 Gauss"
                                Case ChannelTypes.Pressure : .Format = "0 mb"
                                Case ChannelTypes.Radiations : .Format = "0.0 uR/h"
                                Case ChannelTypes.Radon : .Format = "0.0 Bq/m3"
                                Case ChannelTypes.Rain : .Format = "0.0 mm/h"
                                Case ChannelTypes.Strikes : .Format = "0"
                                Case ChannelTypes.Temperature : .Format = "0.000 °C"
                                Case ChannelTypes.WindDirection : .Format = "0.0 deg"
                                Case ChannelTypes.WindSpeed : .Format = "0.0 m/s"
                            End Select
                            If .ChannelType = ChannelTypes.Unused Then
                                MarkErrorLine(i)
                            End If
                        End With
                    Case "nameeng" : .NameENG = l
                    Case "nameita" : .NameITA = l
                    Case "namefra" : .NameFRA = l
                    Case "nameesp" : .NameESP = l
                    Case "namedeu" : .NameDEU = l
                    Case "namejpn" : .NameJPN = l
                    Case "trimzero" : .TrimZero = Val_Single(l)
                    Case "trimslope" : .TrimSlope = Val_Single(l)
                    Case "format" : .Format = l
                    Case "slot" : .Slot = Val_Int(l)
                    Case "end" : Exit For
                    Case Else : MarkErrorLine(i)
                End Select
                ' ------------------------------------------------------------ split format and unit
                Dim sa() As String = Split(.Format, " ")
                If sa.Length = 2 Then
                    .Format = sa(0)
                    .Unit = sa(1)
                End If
            End With
        Next
        For i As Int32 = 1 To NumChannels
            With ChannelArray(i)
                If .NameITA = "" Then .NameITA = .NameENG
                If .NameFRA = "" Then .NameFRA = .NameENG
                If .NameESP = "" Then .NameESP = .NameENG
                If .NameDEU = "" Then .NameDEU = .NameENG
                If .NameJPN = "" Then .NameJPN = .NameENG
            End With
        Next
        If OperatingSystemIsWindows Then
            RTB_RestoreSelectionAndScroll()
            RTB.SelectionLength = 0
            ResumeRedraw(RTB)
        End If
        ' -----------------------------------------------------------------
        Form1.InitControls()
    End Sub

    Private Sub MarkErrorLine(ByVal i As Int32)
        If OperatingSystemIsWindows Then
            RTB_SelectLine(i)
            RTB.SelectionBackColor = Color_ErrorBack
        End If
    End Sub

    Private Function TypeFromString(ByVal s As String) As ChannelTypes
        s = LCase(s)
        If s.Contains("audio") Then Return ChannelTypes.AmbientalNoise
        If s.Contains("dust") Then Return ChannelTypes.Dust
        If s.Contains("umid") Then Return ChannelTypes.Humidity
        If s.Contains("light") Then Return ChannelTypes.Light
        If s.Contains("mag") Then Return ChannelTypes.MagneticField
        If s.Contains("press") Then Return ChannelTypes.Pressure
        If s.Contains("radia") Then Return ChannelTypes.Radiations
        If s.Contains("radon") Then Return ChannelTypes.Radon
        If s.Contains("rain") Then Return ChannelTypes.Rain
        If s.Contains("strik") Or s.Contains("lightning") Then Return ChannelTypes.Strikes
        If s.Contains("temp") Then Return ChannelTypes.Temperature
        If s.Contains("wind") And s.Contains("sp") Then Return ChannelTypes.WindSpeed
        If s.Contains("wind") And s.Contains("dir") Then Return ChannelTypes.WindDirection
        Return ChannelTypes.Unused
    End Function


    ' ===============================================================================================================
    '  RICH TEXT BOX HELPER FUNCTIONS
    ' ===============================================================================================================
    Private Sub RTB_SelectLine(ByVal line As Int32)
        Dim lineStart As Int32 = RTB.GetFirstCharIndexFromLine(line)
        Dim lineLength As Int32 = RTB.Lines(line).Length
        RTB.Select(lineStart, lineLength)
    End Sub

    'Private Function RTB_IsTextselected() As Boolean
    '    Return RTB.SelectedText.Length > 0
    'End Function

    'Private Sub RTB_RemoveEmptyLines()
    '    Dim FirstSelectedLine As Int32 = RTB_GetFirstSelectedLine()
    '    Dim LastSelectedLine As Int32 = RTB_GetLastSelectedLine()
    '    If LastSelectedLine <= FirstSelectedLine Then Return
    '    '
    '    Dim s1 As String
    '    Dim s2 As String
    '    Dim s3 As String = ""
    '    For l As Int32 = FirstSelectedLine To LastSelectedLine
    '        s1 = RTB.Lines(l)
    '        s2 = s1
    '        s2 = Replace(s2, vbTab, "")
    '        s2 = Replace(s2, " ", "")
    '        If s2 <> "" Then
    '            'Debug.Print(Asc(s1).ToString)
    '            s3 &= s1 & vbLf
    '        End If
    '    Next
    '    '
    '    RTB.SelectedText = s3
    '    'Editor_NormalizeAndColorLines(FirstSelectedLine, LastSelectedLine)
    'End Sub

    'Private Function RTB_GetLineLength(ByVal line As Int32) As Int32
    '    If line < 0 Or line > RTB.Lines.Count - 1 Then
    '        Return 0
    '    Else
    '        Return RTB.Lines(line).Length
    '    End If
    'End Function

    'Private Function RTB_GetLineStartIndex(ByVal line As Int32) As Int32
    '    If line < 0 Or line > RTB.Lines.Length - 1 Then
    '        Return 0
    '    Else
    '        Return RTB.GetFirstCharIndexFromLine(line)
    '    End If
    'End Function

    'Private Function RTB_GetWordCount() As Int32
    '    Dim Counter As Int32
    '    Dim Match As System.Text.RegularExpressions.Match
    '    For Each Match In System.Text.RegularExpressions.Regex.Matches(RTB.Text, "\w+")
    '        Counter += 1
    '    Next
    '    Return Counter
    'End Function

    'Private Function RTB_GetFirstSelectedLine() As Int32
    '    Return RTB.GetLineFromCharIndex(RTB.SelectionStart)
    'End Function

    'Private Function RTB_GetLastSelectedLine() As Int32
    '    Return RTB.GetLineFromCharIndex(RTB.SelectionStart + RTB.SelectionLength)
    'End Function

    'Private Sub RTB_SelectLines(ByVal FirstLine As Int32, ByVal LastLine As Int32)
    '    Dim FirstChar As Int32 = RTB.GetFirstCharIndexFromLine(FirstLine)
    '    Dim LastChar As Int32 = RTB.GetFirstCharIndexFromLine(LastLine) + RTB.Lines(LastLine).Length - 1
    '    RTB.Select(FirstChar, LastChar - FirstChar)
    'End Sub


    ' ===============================================================================================================
    '   RTB - SAVE and RESTORE selection and scroll ( scroll aligned with the first text line )
    ' ===============================================================================================================
    Private RTB_OldSelectionStart As Int32
    Private RTB_OldSelectionLength As Int32
    Private RTB_OldFirstChar As Int32
    Private RTB_selection_saved As Int32 = 0
    Private Sub RTB_SaveSelectionAndScroll()
        If RTB_selection_saved <= 0 Then
            RTB_OldSelectionStart = RTB.SelectionStart
            RTB_OldSelectionLength = RTB.SelectionLength
            RTB_OldFirstChar = RTB.GetCharIndexFromPosition(New Point(1, 1))
            RTB_selection_saved = 1
        Else
            RTB_selection_saved += 1
        End If
    End Sub
    Private Sub RTB_RestoreSelectionAndScroll()
        RTB_selection_saved -= 1
        If RTB_selection_saved = 0 Then
            RTB.Select(RTB_OldFirstChar, 0)
            RTB.ScrollToCaret()
            RTB.Select(RTB_OldSelectionStart, RTB_OldSelectionLength)
        End If
    End Sub


End Class